@@ -1,3 +1,5 @@
+mod fibonacci;
+
 use std::convert::TryInto;
 use test_env_log::test;
 
@@ -15,7 +17,7 @@ fn test_string_w_utf8() {
             .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("result")))],
     };
 
-    let program = compile(program, Some(CompileOptions::new())).expect("compile");
+    let program = compile(&program, Some(CompileOptions::new())).expect("compile");
 
     // Compilation was successful
 
@@ -62,7 +64,7 @@ fn test_string_param() {
             ))))],
     };
 
-    let program = compile(cu, None).expect("compile");
+    let program = compile(&cu, None).expect("compile");
 
     vm.run(&program).expect("run");
     let aux = vm.unwrap_aux();
@@ -84,7 +86,7 @@ fn simple_if_statement() {
             ]),
         ],
     };
-    let program = compile(program, Some(CompileOptions::new())).expect("compile");
+    let program = compile(&program, Some(CompileOptions::new())).expect("compile");
 
     // Compilation was successful
 
@@ -109,7 +111,7 @@ fn simple_if_statement_skips_if_false() {
             ]),
         ],
     };
-    let program = compile(program, Some(CompileOptions::new())).unwrap();
+    let program = compile(&program, Some(CompileOptions::new())).unwrap();
 
     // Compilation was successful
 
@@ -143,7 +145,7 @@ fn if_else_test(condition: Card, true_res: Card, false_res: Card, expected_resul
             ]),
         ],
     };
-    let program = compile(program, Some(CompileOptions::new())).expect("compile");
+    let program = compile(&program, Some(CompileOptions::new())).expect("compile");
 
     // Compilation was successful
 
@@ -197,7 +199,7 @@ fn test_local_variable() {
             .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("bar")))],
     };
 
-    let program = compile(program, None).expect("compile");
+    let program = compile(&program, None).expect("compile");
 
     // Compilation was successful
 
@@ -225,7 +227,7 @@ fn local_variable_doesnt_leak_out_of_scope() {
         ],
     };
 
-    let program = compile(program, None).expect("compile");
+    let program = compile(&program, None).expect("compile");
 
     // Compilation was successful
 
@@ -255,7 +257,7 @@ fn simple_while_loop() {
         ],
     };
     /*let program =*/
-    match compile(program, Some(CompileOptions::new())).map_err(|e| e.payload) {
+    match compile(&program, Some(CompileOptions::new())).map_err(|e| e.payload) {
         Ok(_) => {
             panic!("Expected error, update this test pls")
         }
@@ -297,7 +299,7 @@ fn simple_for_loop() {
             ]),
         ],
     };
-    let program = compile(program, Some(CompileOptions::new())).expect("compile");
+    let program = compile(&program, Some(CompileOptions::new())).expect("compile");
 
     // Compilation was successful
 
@@ -321,7 +323,7 @@ fn call_native_test() {
             )))])],
     };
 
-    let prog = compile(cu, CompileOptions::new()).unwrap();
+    let prog = compile(&cu, CompileOptions::new()).unwrap();
 
     struct State {
         called: bool,
@@ -412,7 +414,7 @@ lanes:
           val: "func3"
 "#;
     let cu = serde_yaml::from_str(PROG).unwrap();
-    let prog = compile(cu, CompileOptions::new()).unwrap();
+    let prog = compile(&cu, CompileOptions::new()).unwrap();
 
     vm.run(&prog).expect("run failed");
 
@@ -442,7 +444,7 @@ fn jump_lane_w_params_test() {
         ],
     };
 
-    let program = compile(cu, CompileOptions::new()).expect("compile");
+    let program = compile(&cu, CompileOptions::new()).expect("compile");
 
     let mut vm = Vm::new(()).unwrap();
     vm.run(&program).expect("run");
@@ -460,52 +462,140 @@ fn jump_lane_w_params_test() {
     }
 }
 
-mod fibonacci {
-    use cao_lang::prelude::*;
-    use test_env_log::test;
+#[test]
+fn len_test_empty() {
+    // happy path
+    let cu = CaoIr {
+        lanes: vec![Lane::default()
+            .with_name("main")
+            .with_card(Card::CreateTable)
+            .with_card(Card::Len)
+            .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")))],
+    };
 
-    const RECURSIVE_FIB: &str = include_str!("../benches/fibonacci_program_recursive.yaml");
+    let program = compile(&cu, CompileOptions::new()).expect("compile");
 
-    fn fib(n: i64) -> i64 {
-        let mut a = 0;
-        let mut b = 1;
-        for _ in 0..n {
-            let t = a + b;
-            a = b;
-            b = t;
-        }
-        b
-    }
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
 
-    #[test]
-    fn fibonacci_1() {
-        let cu = serde_yaml::from_str(RECURSIVE_FIB).unwrap();
-        let program = compile(cu, CompileOptions::new()).unwrap();
+    let len = vm
+        .read_var_by_name("g_result", &program.variables)
+        .expect("Failed to read foo variable");
 
-        let mut vm = Vm::new(()).unwrap();
-        vm.stack_push(Value::Integer(1)).unwrap();
-        vm.run(&program).expect("run failed");
+    assert_eq!(len, Value::Integer(0));
+}
 
-        let result = vm
-            .read_var_by_name("result", &program.variables)
-            .expect("Failed to read result variable");
+#[test]
+fn len_test_happy() {
+    // happy path
+    let t = VarNode::from_str_unchecked("t");
+    let cu = CaoIr {
+        lanes: vec![Lane::default()
+            .with_name("main")
+            .with_card(Card::CreateTable)
+            .with_card(Card::SetVar(t))
+            // first property
+            .with_card(Card::ReadVar(t))
+            .with_card(Card::StringLiteral(StringNode("asd".to_string())))
+            .with_card(Card::ScalarInt(IntegerNode(42)))
+            .with_card(Card::SetProperty)
+            // same property as above
+            .with_card(Card::ReadVar(t))
+            .with_card(Card::StringLiteral(StringNode("asd".to_string())))
+            .with_card(Card::ScalarInt(IntegerNode(42)))
+            .with_card(Card::SetProperty)
+            // new property
+            .with_card(Card::ReadVar(t))
+            .with_card(Card::StringLiteral(StringNode("basdasd".to_string())))
+            .with_card(Card::ScalarInt(IntegerNode(42)))
+            .with_card(Card::SetProperty)
+            .with_card(Card::Len)
+            .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")))],
+    };
 
-        assert_eq!(result, Value::Integer(1));
-    }
+    let program = compile(&cu, CompileOptions::new()).expect("compile");
 
-    #[test]
-    fn fibonacci_4() {
-        let cu = serde_yaml::from_str(RECURSIVE_FIB).unwrap();
-        let program = compile(cu, CompileOptions::new()).unwrap();
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
 
-        let mut vm = Vm::new(()).unwrap();
-        vm.stack_push(Value::Integer(4)).unwrap();
-        vm.run(&program).expect("run failed");
+    let len = vm
+        .read_var_by_name("g_result", &program.variables)
+        .expect("Failed to read foo variable");
 
-        let result = vm
-            .read_var_by_name("result", &program.variables)
-            .expect("Failed to read result variable");
+    assert_eq!(len, Value::Integer(2));
+}
 
-        assert_eq!(result, Value::Integer(fib(4)));
-    }
+#[test]
+fn for_each_1() {
+    let cu: CaoIr = serde_yaml::from_str(
+        r#"
+lanes: 
+    - name: main
+      cards:
+        # --- init the example table ---
+        - ty: CreateTable
+        - ty: SetVar
+          val: t
+        - ty: ReadVar
+          val: t
+        - ty: StringLiteral
+          val: f1
+        - ty: ScalarInt
+          val: 1
+        - ty: SetProperty
+        - ty: ReadVar
+          val: t
+        - ty: StringLiteral
+          val: f2
+        - ty: ScalarInt
+          val: 2
+        - ty: SetProperty
+        - ty: ReadVar
+          val: t
+        - ty: StringLiteral
+          val: f3
+        - ty: ScalarInt
+          val: 3
+        - ty: SetProperty
+        - ty: ScalarInt
+          val: 0
+        - ty: SetGlobalVar
+          val: g_result
+        # --- init done ---
+        - ty: ForEach
+          val:
+            lane: 
+              LaneName: pog
+            variable: t
+
+
+    - name: pog
+      arguments:
+        - "key"
+        - "table"
+      cards:
+        - ty: ReadVar
+          val: table
+        - ty: ReadVar
+          val: key
+        - ty: GetProperty
+        - ty: ReadVar
+          val: g_result
+        - ty: Add
+        - ty: SetGlobalVar
+          val: g_result
+        "#,
+    )
+    .unwrap();
+
+    let program = compile(&cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
+
+    let res = vm
+        .read_var_by_name("g_result", &program.variables)
+        .expect("Failed to read result variable");
+
+    assert_eq!(res, Value::Integer(6));
 }
